# Solve systems of linear equations using LUP decomposition.  Taken from
# "Introduction to Algorithms" by Cormen, Leiserson, and Rivest.
#
# Placed in the public domain.
# April 05, 2000
# Neil Schemenauer <nascheme@enme.ucalgary.ca>

def lup_decomposition(a):
	n = len(a)
	pi = range(n)
	for k in range(n-1):
		p = 0.0
		for i in range(k, n):
			if abs(a[i][k]) > p:
				p = abs(a[i][k])
				kp = i
		if p == 0.0:
			raise 'singular matrix'
		pi[kp], pi[k] = pi[k], pi[kp] # exchange
		for i in range(n):
			a[kp][i], a[k][i] = a[k][i], a[kp][i]
		for i in range(k+1, n):
			a[i][k] = a[i][k]/a[k][k]
			for j in range(k+1, n):
				a[i][j] = a[i][j] - a[i][k]*a[k][j]
	return pi

def lup_solve(l, u, pi, b):
	n = len(l)
	y = [0.0]*n
	for i in range(n):
		sum = 0.0
		for j in range(i):
			sum = sum + l[i][j]*y[j]
		y[i] = b[pi[i]] - sum
	x = [0.0]*n
	for i in range(n-1, -1, -1):
		sum = 0.0
		for j in range(i+1, n):
			sum = sum + u[i][j]*x[j]
		x[i] = (y[i] - sum)/u[i][i]
	return x


def test():
	a = [[2.0, 0.0, 2.0, 0.6],
		[3.0, 3.0, 4.0, -2.0],
		[5.0, 5.0, 4.0, 2.0],
		[-1.0, -2.0, 3.4, -1.0]]

	a = [[1.0, 2.0, 0.0],
		[3.0, 5.0, 4.0],
		[5.0, 6.0, 3.0]]

	b = [0.1, 12.5, 10.3]

	pi = lup_decomposition(a)
	print 'a', a
	print 'pi', pi
	x = lup_solve(a, a, pi, b)
	print 'x', x
